module CP where
-- critical pair

import Control.Monad
import Data.Maybe
import Data.List
import Derivation
import E
import ES
import PE
import PES
import Substitution
import Term

cp' :: TermOrder -> PE -> PE -> Position -> Maybe PE
cp' gt rule1 rule2 p = do
  theta <- unify (subtermAt l1 p) l2
  let il1 = substitute l1 theta
  let ir1 = substitute r1 theta
  let il2 = substitute l2 theta
  let ir2 = substitute r2 theta
  guard (not (gt ir1 il1))
  guard (not (gt ir2 il2))
  return (ir1, replace il1 ir2 p)
  where [(l1,r1), (l2,r2)] = PES.rename 0 [rule1, rule2]

cp2 :: TermOrder -> ES -> ES -> [(PE, Derivation)]
cp2 gt es1 es2 = do
-- gt is order > on terms
-- p is non-variable position of l2 where (l2, _) = e2
  E { eqn = e1, eqn_id = i1, active = True } <- es1
  E { eqn = e2, eqn_id = i2, active = True } <- es2
  e1'@(l1, _) <- [ e1, PE.reverse e1 ]
  e2'         <- [ e2, PE.reverse e2 ]
  p <- functionPositions l1
-- if p is root position, then e1' and e2' are not variants.
  guard (p /= [] || not (variant e1' e2'))
  eqn <- maybeToList (cp' gt e1' e2' p)
  return (eqn, CP i1 i2)

-- CP(E) : (extended critical pairs)
cp :: TermOrder -> ES -> [(PE, Derivation)]
cp prec es = nubBy same (cp2 prec es es)
  where same (e1, _) (e2, _) = (variant e1 e2)
