{-# LANGUAGE NamedFieldPuns #-}
module ES where

import Data.List as DL
import Data.Graph
import E
import PE
import PES
import Term


type ES = [E]

showES :: ES -> String
showES es = intercalate "\n" (map show es)

showOTRS :: TermOrder -> ES -> String
showOTRS gt [] = ""
showOTRS gt (E { eqn = (s, t), active } : es)
  | not active = showOTRS gt es
  | s `gt` t = show s ++ " -> " ++ show t ++ "\n" ++
               showOTRS gt es
  | t `gt` s = show t ++ " -> " ++ show s ++ "\n" ++
               showOTRS gt es
  | otherwise = show s ++ " = " ++ show t ++ "\n" ++
                showOTRS gt es

maxId :: ES -> Int
maxId es = maximum [ n | E {eqn_id = n} <- es ]

nextId :: ES -> Int
nextId es = 1 + maxId es 

-- NOTE: ignores non-active equations
toPES :: ES -> PES
toPES es = [ eqn e | e <- es, active e ]

-- checks if variant is in the ES.
inES :: PE -> ES -> Bool
inES _ [] = False
inES pe (e : es) =
  variant pe (eqn e) || inES pe es

sortByWeight :: ES -> ES
sortByWeight es = sortBy comp es 
  where comp (E { eqn = e1 }) (E { eqn = e2 })
          = compare (PE.size e1) (PE.size e2)

sortById :: ES -> ES
sortById es = sortBy comp es
  where comp (E {eqn_id = i1}) (E {eqn_id = i2})
          = compare i1 i2

build :: ES -> Graph
build es = buildG (0, m) edges
  where m = maxId es
        edges = [ ed | e <- es, ed <- depE e]

-- filter irrelevant axioms and lemmata
relevant :: Int -> ES -> ES
relevant goal_id es = [ e | e@(E {eqn_id = i}) <- es, DL.elem i relevant_ids ]
  where g = build es
        relevant_ids = reachable g goal_id
