# Toma
Toma is a theorem prover for first-order equational systems,
the successor of Tofu.
## How to build
1. Install:
- GNU/Make
- Glasgow Haskell Compiler 8.6.5 (or higher)
- Z3 version 4.8.12 (or higher)
2. `$ make` produces the executable `toma`.

## How to run
Toma accepts TPTP format https://www.tptp.org/ .
For example, 

    $ ./toma commutation.p
    ...
    % SZS status Satisfiable

shows the commutation law `x * y = y * x` is not valid in the theory of group.
For another example,

    $ ./toma inverse_unit.p
    ...
    % SZS status Unsatisfiable

shows `e^(-1) = e` is valid in the theory of group.

For the help message, type `$ ./toma -h`.

## How to make StarExec package
To run Toma on StarExec, every binary must be statically linked.
### Build Toma for StarExec
Note that we need an older version of GHC because of a static linking issue. 

0. Install ghcup https://www.haskell.org/ghcup/ .
1. `$ ghcup install ghc 8.0.2`
   
   You may fail here because of lack of libtinfo, etc.
   In that case, install them first. 
2. `$ ghcup set ghc 8.0.2`
3. `$ cabal install --lib parsec # run this command out of toma/ directry`
4. Check if `toma` is produced.

### Build Z3 for StarExec
0. Install cmake.
1. Download Z3 4.8.17 source code (either of the last two in the assets list) from
   https://github.com/Z3Prover/z3/releases/tag/z3-4.8.17 .
2. Extract the archive, and
```
   $ cd z3-4.8.17
   $ mkdir build
   $ cd build
   $ cmake .. -DCMAKE_BUILD_TYPE=Release -DCMAKE_EXE_LINKER_FLAGS:STRING='-static -Wl,--whole-archive -lrt -lpthread -Wl,--no-whole-archive'
   $ make -j4 # compilation takes long time
```
3. Check if `z3` is produced.

### Packaging
For example, we can make the package for CoCo as follows.

0. Make the following directry structure, and copy toma and z3 to the directry.
```
Toma/
├── bin
│   ├── LICENSE_Z3
│   ├── starexec_run_default
│   ├── toma
│   └── z3
└── starexec_description.txt
```
1. `$ cd Toma`
2. `$ zip -r Toma.zip *`
3. Upload `Toma.zip` to StarExec.

## Authors
- Teppei Saito
- Nao Hirokawa
